
/*========================================================================*
 *                                                                        *
 *  XICTOOLS Integrated Circuit Design System                             *
 *  Copyright (c) 1996 Whiteley Research Inc, all rights reserved.        *
 *                                                                        *
 *                                                                        *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      *
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES      *
 *   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-        *
 *   INFRINGEMENT.  IN NO EVENT SHALL STEPHEN R. WHITELEY BE LIABLE       *
 *   FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION      *
 *   OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN           *
 *   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN         *
 *   THE SOFTWARE.                                                        *
 *                                                                        *
 * This software is available for non-commercial use under the terms of   *
 * the GNU General Public License as published by the Free Software       *
 * Foundation; either version 2 of the License, or (at your option) any   *
 * later version.                                                         *
 *                                                                        *
 * A commercial license is available to those wishing to use this         *
 * library or a derivative work for commercial purposes.  Contact         *
 * Whiteley Research Inc., 456 Flora Vista Ave., Sunnyvale, CA 94086.     *
 * This provision will be enforced to the fullest extent possible under   *
 * law.                                                                   *
 *                                                                        *
 * You should have received a copy of the GNU General Public License      *
 * along with this program; if not, write to the Free Software            *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              *
 *========================================================================*
 *                                                                        *
 * Verilog Support Files                                                  *
 *                                                                        *
 *========================================================================*
 $Id: vl_st.h,v 1.3 2004/10/02 22:06:27 stevew Exp $
 *========================================================================*/

#define set_empty() new st_table

struct st_entry
{
    const char *key;
    void *record;
    st_entry *next;
};

class st_table
{
public:
    st_table();
    st_table(int, int, double, int);
    ~st_table();
    bool lookup(const char*, void*);
    bool insert(const char*, void*);
    bool remove(const char**, void*);
    int count() { return (num_entries); }
    friend class st_generator;

    void set_destroy();
    st_table *set_add(const char*);
    st_table *set_eliminate(const char*);
    bool set_find(const char*);
    st_table *set_union(st_table*);
    st_table *set_intersect(st_table*);
    st_table *set_dup();

private:
    void rehash();

    int num_bins;
    int num_entries;
    int max_density;
    int reorder_flag;
    double grow_factor;
    st_entry **bins;
};
typedef st_table *set_t;

class st_generator
{
public:
    st_generator() { }
    st_generator(st_table*);
    bool next(const char**, void*);

protected:
    st_table *table;
    st_entry *entry;
    int index;
};



template<class T> class table_gen : public st_generator
{
public:
    table_gen(void *t) { table = (st_table*)t; entry = 0; index = 0; }
    bool next(const char **sp, T *d)
        { return (st_generator::next(sp, (void*)d)); }
};


template<class T> class table : public st_table
{
public:
    bool lookup(const char *s, T *dp)
        { return (st_table::lookup(s, (void*)dp)); }
    bool insert(const char *s, T d)
        { return (st_table::insert(s, (void*)d)); }
    bool remove(const char **sp, T *dp)
        { return (st_table::remove(sp, (void*)dp)); }
    friend class table_gen<T>;
};


template<class T> void
delete_table(table<T> *tab)
{
    if (!tab)
        return;
    table_gen<T> gen(tab);
    T item;
    const char *key;
    while (gen.next(&key, &item))
        delete item;
    delete tab;
}

